@@ -73,7 +73,7 @@ typedef struct _args_t
     int output_type, n_threads;
 
     char **argv, *output_fname, *targets_list, *regions_list;
-    int argc;
+    int argc, record_cmd_line;
 }
 args_t;
 
@@ -131,7 +131,7 @@ static void init_data(args_t *args)
                 if ( tmp.s ) kputs(" and ", &tmp);
                 kputs("\"IndelGap\"", &tmp);
             }
-            fprintf(pysamerr,"Warning: using %s filter name instead of \"%s\"\n", tmp.s,args->soft_filter);
+            fprintf(pysam_stderr,"Warning: using %s filter name instead of \"%s\"\n", tmp.s,args->soft_filter);
             free(tmp.s);
         }
 
@@ -151,7 +151,7 @@ static void init_data(args_t *args)
         }
     }
 
-    bcf_hdr_append_version(args->hdr, args->argc, args->argv, "bcftools_filter");
+    if (args->record_cmd_line) bcf_hdr_append_version(args->hdr, args->argc, args->argv, "bcftools_filter");
 
     if ( args->filter_str )
         args->filter = filter_init(args->hdr, args->filter_str);
@@ -400,26 +400,27 @@ static void set_genotypes(args_t *args, bcf1_t *line, int pass_site)
 
 static void usage(args_t *args)
 {
-    fprintf(pysamerr, "\n");
-    fprintf(pysamerr, "About:   Apply fixed-threshold filters.\n");
-    fprintf(pysamerr, "Usage:   bcftools filter [options] <in.vcf.gz>\n");
-    fprintf(pysamerr, "\n");
-    fprintf(pysamerr, "Options:\n");
-    fprintf(pysamerr, "    -e, --exclude <expr>          exclude sites for which the expression is true (see man page for details)\n");
-    fprintf(pysamerr, "    -g, --SnpGap <int>            filter SNPs within <int> base pairs of an indel\n");
-    fprintf(pysamerr, "    -G, --IndelGap <int>          filter clusters of indels separated by <int> or fewer base pairs allowing only one to pass\n");
-    fprintf(pysamerr, "    -i, --include <expr>          include only sites for which the expression is true (see man page for details\n");
-    fprintf(pysamerr, "    -m, --mode [+x]               \"+\": do not replace but add to existing FILTER; \"x\": reset filters at sites which pass\n");
-    fprintf(pysamerr, "    -o, --output <file>           write output to a file [standard output]\n");
-    fprintf(pysamerr, "    -O, --output-type <b|u|z|v>   b: compressed BCF, u: uncompressed BCF, z: compressed VCF, v: uncompressed VCF [v]\n");
-    fprintf(pysamerr, "    -r, --regions <region>        restrict to comma-separated list of regions\n");
-    fprintf(pysamerr, "    -R, --regions-file <file>     restrict to regions listed in a file\n");
-    fprintf(pysamerr, "    -s, --soft-filter <string>    annotate FILTER column with <string> or unique filter name (\"Filter%%d\") made up by the program (\"+\")\n");
-    fprintf(pysamerr, "    -S, --set-GTs <.|0>           set genotypes of failed samples to missing (.) or ref (0)\n");
-    fprintf(pysamerr, "    -t, --targets <region>        similar to -r but streams rather than index-jumps\n");
-    fprintf(pysamerr, "    -T, --targets-file <file>     similar to -R but streams rather than index-jumps\n");
-    fprintf(pysamerr, "        --threads <int>           number of extra output compression threads [0]\n");
-    fprintf(pysamerr, "\n");
+    fprintf(pysam_stderr, "\n");
+    fprintf(pysam_stderr, "About:   Apply fixed-threshold filters.\n");
+    fprintf(pysam_stderr, "Usage:   bcftools filter [options] <in.vcf.gz>\n");
+    fprintf(pysam_stderr, "\n");
+    fprintf(pysam_stderr, "Options:\n");
+    fprintf(pysam_stderr, "    -e, --exclude <expr>          exclude sites for which the expression is true (see man page for details)\n");
+    fprintf(pysam_stderr, "    -g, --SnpGap <int>            filter SNPs within <int> base pairs of an indel\n");
+    fprintf(pysam_stderr, "    -G, --IndelGap <int>          filter clusters of indels separated by <int> or fewer base pairs allowing only one to pass\n");
+    fprintf(pysam_stderr, "    -i, --include <expr>          include only sites for which the expression is true (see man page for details\n");
+    fprintf(pysam_stderr, "    -m, --mode [+x]               \"+\": do not replace but add to existing FILTER; \"x\": reset filters at sites which pass\n");
+    fprintf(pysam_stderr, "        --no-version              do not append version and command line to the header\n");
+    fprintf(pysam_stderr, "    -o, --output <file>           write output to a file [standard output]\n");
+    fprintf(pysam_stderr, "    -O, --output-type <b|u|z|v>   b: compressed BCF, u: uncompressed BCF, z: compressed VCF, v: uncompressed VCF [v]\n");
+    fprintf(pysam_stderr, "    -r, --regions <region>        restrict to comma-separated list of regions\n");
+    fprintf(pysam_stderr, "    -R, --regions-file <file>     restrict to regions listed in a file\n");
+    fprintf(pysam_stderr, "    -s, --soft-filter <string>    annotate FILTER column with <string> or unique filter name (\"Filter%%d\") made up by the program (\"+\")\n");
+    fprintf(pysam_stderr, "    -S, --set-GTs <.|0>           set genotypes of failed samples to missing (.) or ref (0)\n");
+    fprintf(pysam_stderr, "    -t, --targets <region>        similar to -r but streams rather than index-jumps\n");
+    fprintf(pysam_stderr, "    -T, --targets-file <file>     similar to -R but streams rather than index-jumps\n");
+    fprintf(pysam_stderr, "        --threads <int>           number of extra output compression threads [0]\n");
+    fprintf(pysam_stderr, "\n");
     exit(1);
 }
 
@@ -432,6 +433,7 @@ int main_vcffilter(int argc, char *argv[])
     args->output_fname = "-";
     args->output_type = FT_VCF;
     args->n_threads = 0;
+    args->record_cmd_line = 1;
     int regions_is_file = 0, targets_is_file = 0;
 
     static struct option loptions[] =
@@ -450,6 +452,7 @@ int main_vcffilter(int argc, char *argv[])
         {"threads",required_argument,NULL,9},
         {"SnpGap",required_argument,NULL,'g'},
         {"IndelGap",required_argument,NULL,'G'},
+        {"no-version",no_argument,NULL,8},
         {NULL,0,NULL,0}
     };
     char *tmp;
@@ -490,6 +493,7 @@ int main_vcffilter(int argc, char *argv[])
                 else error("The argument to -S not recognised: %s\n", optarg);
                 break;
             case  9 : args->n_threads = strtol(optarg, 0, 0); break;
+            case  8 : args->record_cmd_line = 0; break;
             case 'h':
             case '?': usage(args);
             default: error("Unknown argument: %s\n", optarg);
