@@ -25,6 +25,8 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.  */
 
+#include <config.h>
+
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
@@ -35,6 +37,7 @@ DEALINGS IN THE SOFTWARE.  */
 #include "htslib/kstring.h"
 #include "kprobaln.h"
 #include "sam_opts.h"
+#include "samtools.h"
 
 #define USE_EQUAL 1
 #define DROP_TAG  2
@@ -115,7 +118,7 @@ void bam_fillmd1_core(bam1_t *b, char *ref, int ref_len, int flag, int max_nm)
         if (old_nm) old_nm_i = bam_aux2i(old_nm);
         if (!old_nm) bam_aux_append(b, "NM", 'i', 4, (uint8_t*)&nm);
         else if (nm != old_nm_i) {
-            fprintf(pysamerr, "[bam_fillmd1] different NM for read '%s': %d -> %d\n", bam_get_qname(b), old_nm_i, nm);
+            fprintf(pysam_stderr, "[bam_fillmd1] different NM for read '%s': %d -> %d\n", bam_get_qname(b), old_nm_i, nm);
             bam_aux_del(b, old_nm);
             bam_aux_append(b, "NM", 'i', 4, (uint8_t*)&nm);
         }
@@ -133,7 +136,7 @@ void bam_fillmd1_core(bam1_t *b, char *ref, int ref_len, int flag, int max_nm)
                 if (i < str->l) is_diff = 1;
             } else is_diff = 1;
             if (is_diff) {
-                fprintf(pysamerr, "[bam_fillmd1] different MD for read '%s': '%s' -> '%s'\n", bam_get_qname(b), old_md+1, str->s);
+                fprintf(pysam_stderr, "[bam_fillmd1] different MD for read '%s': '%s' -> '%s'\n", bam_get_qname(b), old_md+1, str->s);
                 bam_aux_del(b, old_md);
                 bam_aux_append(b, "MD", 'Z', str->l + 1, (uint8_t*)str->s);
             }
@@ -207,7 +210,7 @@ int bam_cap_mapQ(bam1_t *b, char *ref, int ref_len, int thres)
     if (t > thres) return -1;
     if (t < 0) t = 0;
     t = sqrt((thres - t) / thres) * thres;
-//  fprintf(pysamerr, "%s %lf %d\n", bam_get_qname(b), t, q);
+//  fprintf(pysam_stderr, "%s %lf %d\n", bam_get_qname(b), t, q);
     return (int)(t + .499);
 }
 
@@ -333,7 +336,7 @@ int bam_prob_realn(bam1_t *b, const char *ref)
 }
 
 int calmd_usage() {
-    fprintf(pysamerr,
+    fprintf(pysam_stderr,
 "Usage: samtools calmd [-eubrAES] <aln.bam> <ref.fasta>\n"
 "Options:\n"
 "  -e       change identical bases to '='\n"
@@ -344,18 +347,18 @@ int calmd_usage() {
 "  -r       compute the BQ tag (without -A) or cap baseQ by BAQ (with -A)\n"
 "  -E       extended BAQ for better sensitivity but lower specificity\n");
 
-    sam_global_opt_help(pysamerr, "-....");
+    sam_global_opt_help(pysam_stderr, "-....");
     return 1;
 }
 
 int bam_fillmd(int argc, char *argv[])
 {
     int c, flt_flag, tid = -2, ret, len, is_bam_out, is_uncompressed, max_nm, is_realn, capQ, baq_flag;
-    samFile *fp, *fpout = 0;
-    bam_hdr_t *header;
-    faidx_t *fai;
-    char *ref = 0, mode_w[8], *ref_file;
-    bam1_t *b;
+    samFile *fp = NULL, *fpout = NULL;
+    bam_hdr_t *header = NULL;
+    faidx_t *fai = NULL;
+    char *ref = NULL, mode_w[8], *ref_file;
+    bam1_t *b = NULL;
     sam_global_args ga = SAM_GLOBAL_ARGS_INIT;
 
     static const struct option lopts[] = {
@@ -382,7 +385,7 @@ int bam_fillmd(int argc, char *argv[])
         case 'A': baq_flag |= 1; break;
         case 'E': baq_flag |= 2; break;
         default:  if (parse_sam_global_opt(c, optarg, lopts, &ga) == 0) break;
-            fprintf(pysamerr, "[bam_fillmd] unrecognized option '-%c'\n\n", c);
+            fprintf(pysam_stderr, "[bam_fillmd] unrecognized option '-%c'\n\n", c);
             /* else fall-through */
         case '?': return calmd_usage();
         }
@@ -393,35 +396,51 @@ int bam_fillmd(int argc, char *argv[])
     if (optind + (ga.reference == NULL) >= argc)
         return calmd_usage();
     fp = sam_open_format(argv[optind], "r", &ga.in);
-    if (fp == 0) return 1;
-
-    header = sam_hdr_read(fp);
-    if (header == NULL || header->n_targets == 0) {
-        fprintf(pysamerr, "[bam_fillmd] input SAM does not have header. Abort!\n");
+    if (fp == NULL) {
+        print_error_errno("calmd", "Failed to open input file '%s'", argv[optind]);
         return 1;
     }
 
-    fpout = sam_open_format("-", mode_w, &ga.out);
-    sam_hdr_write(fpout, header);
+    header = sam_hdr_read(fp);
+    if (header == NULL || header->n_targets == 0) {
+        fprintf(pysam_stderr, "[bam_fillmd] input SAM does not have header. Abort!\n");
+        goto fail;
+    }
+    
+    fpout = sam_open_format(pysam_stdout_fn, mode_w, &ga.out);
+    if (fpout == NULL) {
+        print_error_errno("calmd", "Failed to open output");
+        goto fail;
+    }
+    if (sam_hdr_write(fpout, header) < 0) {
+        print_error_errno("calmd", "Failed to write sam header");
+        goto fail;
+    }
 
     ref_file = argc > optind + 1 ? argv[optind+1] : ga.reference;
     fai = fai_load(ref_file);
 
     if (!fai) {
-        perror(ref_file);
-        return 1;
+        print_error_errno("calmd", "Failed to open reference file '%s'", ref_file);
+        goto fail;
     }
 
     b = bam_init1();
+    if (!b) {
+        fprintf(pysam_stderr, "[bam_fillmd] Failed to allocate bam struct\n");
+        goto fail;
+    }
     while ((ret = sam_read1(fp, header, b)) >= 0) {
         if (b->core.tid >= 0) {
             if (tid != b->core.tid) {
                 free(ref);
                 ref = fai_fetch(fai, header->target_name[b->core.tid], &len);
                 tid = b->core.tid;
-                if (ref == 0)
-                    fprintf(pysamerr, "[bam_fillmd] fail to find sequence '%s' in the reference.\n",
+                if (ref == 0) { // FIXME: Should this always be fatal?
+                    fprintf(pysam_stderr, "[bam_fillmd] fail to find sequence '%s' in the reference.\n",
                             header->target_name[tid]);
+                    if (is_realn || capQ > 10) goto fail; // Would otherwise crash
+                }
             }
             if (is_realn) bam_prob_realn_core(b, ref, len, baq_flag);
             if (capQ > 10) {
@@ -430,7 +449,14 @@ int bam_fillmd(int argc, char *argv[])
             }
             if (ref) bam_fillmd1_core(b, ref, len, flt_flag, max_nm);
         }
-        sam_write1(fpout, header, b);
+        if (sam_write1(fpout, header, b) < 0) {
+            print_error_errno("calmd", "failed to write to output file");
+            goto fail;
+        }
+    }
+    if (ret < -1) {
+        fprintf(pysam_stderr, "[bam_fillmd] Error reading input.\n");
+        goto fail;
     }
     bam_destroy1(b);
     bam_hdr_destroy(header);
@@ -438,6 +464,18 @@ int bam_fillmd(int argc, char *argv[])
     free(ref);
     fai_destroy(fai);
     sam_close(fp);
-    sam_close(fpout);
+    if (sam_close(fpout) < 0) {
+        fprintf(pysam_stderr, "[bam_fillmd] error when closing output file\n");
+        return 1;
+    }
     return 0;
+
+ fail:
+    free(ref);
+    if (b) bam_destroy1(b);
+    if (header) bam_hdr_destroy(header);
+    if (fai) fai_destroy(fai);
+    if (fp) sam_close(fp);
+    if (fpout) sam_close(fpout);
+    return 1;
 }
